#ifndef __CSTMTYPES_H__
#define __CSTMTYPES_H__
#include <PalmOS.h>
#include <NetMgr.h>
#include <VFSMgr.h>
#include <FileStream.h>
#include <ExgMgr.h>
#include <DataMgr.h>
#include <procid.h>

#define nameChunkSize                   256
#define pathChunkSize                   4096
#define maxDwnldSlots                   4
#define maxTotalThreads                 4
#define maxHandledMemoryChunks          50

#define errTypeOfErrWarning             1
#define errTypeOfErrMemory              2
#define errTypeOfErrInput               4
#define errTypeOfErrOutput              8
#define errTypeOfErrAlgoritm            16
#define errTypeOfErrRecovered           32
#define errTypeOfErrVisible             64  //is visible to user
#define errTypeOfErrCritical            128
#define errTypeOfErrMemoryCritical      (errTypeOfErrMemory|errTypeOfErrCritical)

#define cstmMemTypeOfChnkPtr            0
#define cstmMemTypeOfChnkHandle         1
#define cstmMemTypeOfChnkCached         2
#define cstmMemTypeOfChnkLocked         4
#define cstmMemTypeOfChnkHandleLocked   (cstmMemTypeOfChnkHandle|cstmMemTypeOfChnkLocked)
#define cstmMemTypeOfChnkReserve        8
#define cstmMemTypeOfChnkAll            (cstmMemTypeOfChnkHandle|cstmMemTypeOfChnkCached|cstmMemTypeOfChnkReserve)
#define cstmMemTypeOfChnkAllLocked      (cstmMemTypeOfChnkAll|cstmMemTypeOfChnkLocked)
#define cstmMemTypeOfChnkNeedsUpdate    16
#define cstmMemTypeOfChnkSecureUpdate   32  //for Unlocked by MemMngr will generate
                                             //type cstmMemTypeOfChnkNeedsUpdate
#define cstmMemTypeOfChnkNONUpdatable   64  //for system handled chunks
#define cstmMemTypeOfChnkCritical       128
#define cstmMemTypeOfChnkMaskAll        255

#define heapIDDynamic                   0
#define heapIDStorage                   1

#define paramTypeMemAllocs              1
#define paramTypeMemOper                2
#define paramTypeFlsOper                3
#define paramTypeHttpRsp                4

//------------------------------------------------------------------------------
#define flsRuleNone           0
#define flsRuleDelExists      1
#define flsRuleARenExists     2
#define flsRuleExists         flsRuleDelExists|flsRuleARenExists
#define flsRulePreAlloc       4
#define flsRuleChckFrSpace    8
#define flsRuleCnfgEnabled    16
#define flsRuleLoggingEnabled 32
#define flsRuleAutoRedirect   64

/*---------------------------------------------------------------------------*/
#define errMemManTableFull              1
#define errMemManRecoveryFailed         2
#define errMemManIDNotExist             3
#define errMemManIncorrectTypeLock      4
#define errMemManIncorrectTypeUnlock    5
#define errMemManNONUpdatableAccessed   6
#define errMemManPtrRangeIncorrect      7
#define errMemManAllocFailed            8
#define errMemManNotHandleType          9
#define errMemManPtrInvalid             10

#define errFlsWriteFailed               16
#define errFlsVfsResizeFailed           17
#define errFlsPathNameFailed            18
#define errFlsExistCmdFailed            19
#define errFlsVfsCreateFailed           20
#define errFlsVfsOpenFailed             21
#define errFlsFsOpenFailed              22
#define errFlsVfsCloseFailed            23
#define errFlsFsCloseFailed             24
#define errFlsDbRenameFailed            25
#define errFlsDbConvFailed              26
#define errFlsFsDelFailed               27
#define errFlsVfsDelFailed              28
#define errFlsFsResizeFailed            29
#define errFlsFsSeekFailed              30
#define errFlsVfsSeekFailed             31

#define errHttpNotHTTP1_1               36
#define errHttpRqstFailed               37
#define errHttpScktOpenFailed           38
#define errHttpNotReady2Go              39
#define errHttpSendFailed               40
#define errHttpRcvFailed                41
#define errHttpOpenNetLibFailed         42
#define errHttpCloseNetLibFailed        43
#define errHttpForceCloseNetLibFailed   44
#define errHttpCloseFailed              45
#define errHttpHostNotFound             46
#define errHttpFailedConnect            47
#define errHttpRcvBufferFailed          48

#define errBufferIsFree                 52
#define errBufferIsFull                 53
/*---------------------------------------------------------------------------*/

//#define 
//#define 
//#define 
//#define 
//#define 

typedef enum{
//encode
cEsc1='{', cEsc2='}', cEsc3='|', cEsc4=0x5C/*backslash <\> */, cEsc5='^', 
cEsc6='[', cEsc7=']',//ipv6 in authority not encoded
cEsc8='`', cEsc9='<', cEscA='>', cEscB=0x22/*quatation <">*/, cFragment='#', 
cPercent='%',//partial encode
lowerCode=0x21, upperCode=0x7E,

//not encoded
cDot='.', cMin='-',cLLine='_',cTilde='~',

cSlash='/',cQstn='?',
cx2Dot=':', cNum='#', cComAt='@',

cExlam='!',cDollar='$',cApostr=0x27/*apostrophe <'>*/,
cLParen='(',cRParen=')',cAster='*',cPlus='+',cComma=',',
cSemicol=';',cEqual='=',
}toEscape;
// "{" | "}" | "|" | "\" | "^" | "[" | "]" | "`" 
// "<" | ">" | "#" | "%" | <">

// ":" / "/" / "?" / "#" / "[" / "]" / "@" - gen-delims
// "!" / "$" / "&" / "'" / "(" / ")" / "*" / "+" / "," / ";" / "=" - sub-delims
// "-" / "." / "_" / "~" - can be present
// better not
// RFC 2396, RFC 3986

typedef struct{
   MemPtr basePtr;//this is start of memory chunk where url is
   MemPtr authorityPtr;//major component
   MemPtr usrInfoPtr;//sub component of authority
   MemPtr hostPtr;//sub component of authority
   MemPtr pathPtr;//major component
   MemPtr filePtr;
   MemPtr queryPtr;//major component
   MemPtr fragmPtr;//major component
   UInt16 fullUrlLen;
   UInt16 usrInfoLen;
   UInt16 hostLen;
   UInt16 pathLen;
   UInt16 flNameLen;
   UInt16 queryLen;
   UInt16 fragmLen;
   UInt16 srvrPort;
   Int16 scheme;
   NetSocketRef openedSocket;
   NetSocketAddrINType httpSrvrAddr;
}urlInfoDataType;
typedef urlInfoDataType *urlInfoDataPtr;

typedef struct{
   UInt32 fullDataLen;
   UInt32 rcvdDataLen;
   UInt16 offstName;
   UInt16 crntChnk;
   UInt8 flNmLen;
   UInt8 numChnks;
   UInt32 chnkBase;//pointer to start from,=fullDataLen<<1
   UInt32 chnk0B[maxTotalThreads];
   UInt32 chnk0E[maxTotalThreads];
   union{
    FileRef flR2Write2;
    FileHand flH2Write2;
   }destRef;
   UInt32 id4Mem4PathName;
}appFileDataInfoType;
typedef appFileDataInfoType *appFileDataInfoPtr;

typedef struct{
   Char ghCacheControl[64];
   Char ghConnection[64];
   Char ghDate[64];
   Char ghPragma[64];
   Char ghTrailer[64];
   Char ghTransferEncoding[64];
   Char ghUpgrade[64];
   Char ghVia[64];
   Char ghWarning[64];
   UInt16 ghMask;
}ghInfoType;
typedef ghInfoType *ghInfoPtr;

typedef struct{
   Char rqhAccept[64];
   Char rqhAcceptCharset[64];
   Char rqhAcceptEncoding[64];
   Char rqhAcceptLanguage[64];
   Char rqhAuthorization[128];
   Char rqhExpect[64];
   Char rqhFrom[128];
   Char rqhHost[128];
   Char rqhIfMatch[64];
   Char rqhIfModifiedSince[64];
   Char rqhIfNoneMatch[64];
   Char rqhIfRange[64];
   Char rqhIfUnmodifiedSince[64];
   Char rqhMaxForwards[64];
   Char rqhProxyAuthorization[128];
   Char rqhRange[64];
   Char rqhReferer[4096];
   Char rqhTE[64];
   Char rqhUseAgent[64];
   UInt32 rqhMask;
}rqhInfoType;
typedef rqhInfoType *rqhInfoPtr;

typedef struct{
   Char rshAcceptRanges[64];
   Char rshAge[64];
   Char rshETag[64];
   Char rshLocation[4096];
   Char rshProxyAuthenticate[128];
   Char rshRetryAfter[64];
   Char rshServer[64];
   Char rshVary[64];
   Char rshWWWAuthenticate[256];
   UInt16 rshMask;
}rshInfoType;
typedef rshInfoType *rshInfoPtr;

typedef struct{
   Char ehAllow[64];
   Char ehContentEncoding[64];
   Char ehContentLanguage[64];
   Char ehContentLength[32];
   Char ehContentLocation[4096];
   Char ehContentMD5[64];
   Char ehContentRange[64];
   Char ehContentType[64];
   Char ehExpires[64];
   Char ehLastModified[64];
   Char ehExtOne[256];//!!
   Char ehExtTwo[256];//!!
   UInt32 ehMask;
}ehInfoType;
typedef ehInfoType *ehInfoPtr;

typedef struct{
   MemPtr strNamePtr;
   Int16 strLen;
}str4NameType;
typedef str4NameType *str4NamePtr;

typedef struct{
   Char elmnt[256];
   UInt8 elmntLen;
}elmntType;
typedef elmntType *elmntPtr;

typedef enum{
   sCr=0x0D,
   sLf=0x0A
}toCrLf;
typedef NetSocketAddrINType *nSAINTPtr;//to add manualy

typedef struct{
   UInt32 cCmp[4];
   Char cCompare;
   Char *rscCh;
   UInt8 minusIsLoc;
}setCmpType;

typedef struct{
   UInt32 chnkSizeNeeded;
   MemHandle toSetH;
   MemPtr toSetP;
}memAllocs;//paramType=1

typedef struct{
   UInt32 uniqueID;
   UInt8 typeOfChnk;
}memOper;//=2

typedef struct{
   UInt32 param_0;
   UInt32 param_1;
}flsOper;//=3

typedef struct{
   UInt16 rspCode;
}httpAnswer;//=4

typedef struct{
   UInt16 procID;
   UInt16 procCrtclStep;
   UInt32 typeOfErr;/*typeOfErr- will be set by proc
    0b-warning, 1b- memErr, 2b- inErr, 3b- outErr, 4b- algErr,
    5b-recovered,6b-visible for user, 7b- initErr/crtcl/fatal err*/
   UInt16 dwnldID;
   UInt16 threadID;
   UInt8 numOfErrs;
   Err arryOfErrs[8];
   UInt8 paramType;
   union{
    UInt32 params4Err[4];//used union to pass named vars
    memAllocs paramsMemAlloc;
    memOper paramsMemOper;
    flsOper paramsFlsOper;
    httpAnswer paramsHttpAnswer;
   }vars4Err;
}appErrType;
typedef appErrType *appErrPtr;

typedef struct{
   MemHandle thisChunk;
   UInt8 errPosFree;
   appErrType bufferOfErrs[64];//0-active,1-62 passed errors, 63-reserved for errBuffer
}appErrBufferType;
typedef appErrBufferType *appBufferErrPtr;

typedef struct{
   UInt32 uniqueID;
   UInt16 procID;
   UInt16 procCrtclStep;
   UInt8 typeOfChnk;/*0b- memPtrNew=0 MemHandleNew=1, 1b- cached,2b-locked,
   3b- reserve, 4b-needs update, 5b-secure update,6b-nonupdatable,
   7b-crtcl allocation*/
   UInt8 lockCounter;
   UInt32 uniqueIDlink;//link to reserve
   MemHandle newHandleConst;
   MemHandle oldHandleConst;
   MemPtr newPtrConst;
   MemPtr oldPtrConst;
   UInt32 sizeInUse;
   UInt32 sizeOld;
}chnkDataType;
typedef chnkDataType *chnkDataPtr;

typedef struct{
   UInt32 nextUniqueID;
   UInt32 uniqueIDHigh;
   UInt32 chnksAllocated;
   UInt32 nextChnkFree;//=0 no freespace
   UInt32 numOfChnkDataAlloc;
   MemHandle chnkDataHandle;
   chnkDataPtr chnkDataLocked;
   chnkDataPtr chnkDataRelative;//after allocations pointer is set to chunk data
}appMemTableType;

typedef struct{
   UInt32 memorySize;
   UInt32 memoryFree;
   UInt32 memoryChnkFree;
}memoryInfoType;

typedef struct{
  UInt32 id4Mem4UrlHandle;
  UInt32 id4Mem4UrlOriginal;
  UInt32 id4Mem4GhMem;
  UInt32 id4Mem4RqhMem;
  UInt32 id4Mem4RshMem;
  UInt32 id4Mem4EhMem;
  UInt32 id4Mem4Head;
  UInt32 id4Mem4Get;
  UInt32 currentIP;
  UInt16 memStatus;//init=0
  ghInfoPtr ghMemPtr;
  rqhInfoPtr rqhMemPtr;
  rshInfoPtr rshMemPtr;
  ehInfoPtr ehMemPtr;
  MemPtr ptrUrl;
  MemPtr hdPtr;
  MemPtr gtPtr;
}httpConstDataType;
typedef httpConstDataType *httpConstDataPtr;

typedef struct{
 
}httpSendMthd;

typedef struct{
}httpActiveVarsDataType;

typedef struct{
  UInt16 threadStt;//threadStt, same as dwnldStt
  UInt16 threadID;//!=0 links to chunk data of file
  UInt8 link2Chnk;//on vfsfs
  urlInfoDataType urlInfo;
  httpConstDataType httpConst;
  UInt16 rspCode;//=0
  UInt16 httpStatus;//=0
  UInt16 httpCstm;//=0
  UInt16 httpDscr;//=0
  Int16 hdGotLen;//=0,len of header of http request//remove??
  UInt16 uiState;//per thread state
  union{
   UInt8 dummy;
  }stageParam;
  UInt32 chnkdWrt;
  UInt32 chnkLeft;
  UInt8 httpMethod;
  UInt8 threadDscr;
/* threadDscr, 0b-main thread,1b-mirrored thread,
 * 2b-scheme is used(Url is set),3b-is redirected,4b-was redirected,
 * 5b-has level set(not implemented)
 */
  UInt16 threadIDMain;//always main threadID
  UInt16 maxThreads;//set to number of succesful connection
}srcDstHttpDataType;
typedef srcDstHttpDataType *srcDstHttpDataPtr;

typedef struct{
 UInt8 curVfsNum;//=0
 UInt8 notRam;//=0
 UInt8 flsCstm;//=0
 UInt8 flsRules;//=0
 UInt16 flsStatus;//=0
 UInt16 flsDscr;//=0
 appFileDataInfoType mainFile;
 str4NameType path4CmnUse;
 str4NameType name4ViewUse;
}srcDstVfsFsDataType;

typedef struct{
 UInt8 maxThreads;//not implemented
 UInt8 numThreads;//=1, total threads 
 UInt8 usedThreads;
 UInt8 actvThreads;//number of threads currently working
 UInt8 dwnldDscr;
/* dwnldDscr, 0b-has main mirror threads,1b-has mirrored threads,
 * 2b-scheme is used(Url is set),3b-has redirected thread(s),
 * 4b-had redirected thread(s)
 * mostly a copy of threadDscr
 */
 UInt16 dwnldStt;
/* dwnldStt, 0b-used(can be 0 if numThreads=0),1b-active,
 * 2b-ready(all data is present, to become active)
 * 3b-paused,4b-stopped,[3,4]b-queued,5b-complete
 * 6b-not a full state,7b-error,8b-check state*/
 UInt16 actvThread;//threadID
 UInt16 threadQueue[maxTotalThreads];//=0 thread IDs
 UInt16 dwnldID;
 UInt16 btnState;
 UInt16 uiUpdateCache;
 /*uiUpdateCache 0b-set default,1b-server IP,2b-filename,
  * 3b-size downloaded/fullsize,4b-status info('error','downloading',..),
  * 5b-use info field,6b-update buttons,7b-show err httpStatus,rspCode,other
  * [1..3]b are treated as permanent, all other are cleared before applied*/
 UInt16 uiState;//uiUpdateCache, applied
 srcDstVfsFsDataType fileDst;
 UInt32 id4Mem4SrcDstHttpData;//allocated=numThreads*sizeof(srcDstHttpDataType)
 srcDstHttpDataPtr httpSrc;//threads
}srcDstDataType;
typedef srcDstDataType *srcDstDataPtr;

typedef struct{
 UInt16 actvSlot;//=0
 UInt8 actvSlots;
 UInt8 usedSlots;
 UInt8 totalSlots;
 UInt8 actvThreads;
 UInt8 maxThreads;
 UInt16 brwsSlot;//also used as current slot under processing
 UInt16 brwsThread;//same use as for slot
 UInt16 visSlot;
 UInt16 visThread;
 UInt16 dwnldQueue[maxDwnldSlots];//=0 dwnldIDs
 UInt32 id4Mem4TempData;
 srcDstDataType dwnldData[maxDwnldSlots];
}srcDstListType;
typedef srcDstListType *srcDstListPtr;

#define sizeOfChnkDataType sizeof(chnkDataType)
#define appCreator  'Dman'
#define typeFDef    'undf'
#define typeFTDef   'temp'
#define typeFConf   'conf'
#endif